(*----------------------------------------------------------------------------*)
(* Favnir ... Export of Microsoft Internet Explorer Favorites (Bookmarks)     *)
(* Author: Joachim Pimiskern  7. Juni 2002                                    *)
(* Freeware; use and/or midify it at your own risk.                           *)
(* Version: 1.0                                                               *)
(*----------------------------------------------------------------------------*)
unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, FileCtrl, ComCtrls;

type

  TFormMain = class(TForm)
    LabelDir : TLabel;
    ButtonStart: TButton;
    ButtonExit: TButton;
    ButtonSaveAs: TButton;
    SaveDialog: TSaveDialog;
    ButtonSearchFolder: TButton;
    PageControl: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    MemoHeader: TMemo;
    MemoFooter: TMemo;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    MemoFolderHeader: TMemo;
    MemoFolderFooter: TMemo;
    Memo: TTabSheet;
    MemoUrlData: TMemo;
    Label2: TLabel;
    Label1: TLabel;
    TabSheet5: TTabSheet;
    CheckBoxEncode: TCheckBox;
    ListBoxSchemes: TListBox;
    ButtonAddScheme: TButton;
    ButtonDeleteScheme: TButton;
    ButtonSaveScheme: TButton;
    LabelCurrentScheme: TLabel;
    Label3: TLabel;
    procedure ButtonExitClick(Sender: TObject);
    procedure ButtonStartClick(Sender: TObject);
    procedure ButtonSaveAsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ButtonSearchFolderClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure CheckBoxEncodeClick(Sender: TObject);
    procedure ButtonAddSchemeClick(Sender: TObject);
    procedure ButtonSaveSchemeClick(Sender: TObject);
    procedure ListBoxSchemesClick(Sender: TObject);
    procedure ButtonDeleteSchemeClick(Sender: TObject);
  private
    breakFlag: boolean;
  protected
    FavoritesDirectory: string;
    sl       : TStringList;
    r        : TStringList;
    encode   : boolean;
    scheme   : string;
    function  encodeString(s: string): string;
    function  path0(s: string): string;
    function  path1(s: string): string;
    function  path2(s: string): string;
    procedure ProcessUrl(path,s: string);
    procedure ProcessDir(path,title: string);
    procedure ReadIniFile;
    procedure WriteIniFile;
    procedure DeleteScheme;
    function  GetIniFileName: string;
  public
  end;

var
  FormMain: TFormMain;

implementation
uses
   ShellAPI, ShlObj, inifiles;

{$R *.DFM}

(*----------------------------------------------------------------------------*)
(* Aus der Delphi-Fundgrube von Simon Reinhardt                               *)
(* http://www.delphi-fundgrube.de/files/shdemo.txt                            *)
(* http://www.delphi-fundgrube.de                                             *)
(*----------------------------------------------------------------------------*)
function GetFolder(Root:integer;Caption:string):string;
var
    bi            : TBROWSEINFO;
    lpBuffer      : PChar;
    pidlPrograms,
    pidlBrowse    : PItemIDList;
begin
  if (not SUCCEEDED(SHGetSpecialFolderLocation(getactivewindow,Root,pidlPrograms))) then
    exit;

  lpBuffer := StrAlloc(max_path);
  bi.hwndOwner := getactivewindow;
  bi.pidlRoot  := pidlPrograms;
  bi.pszDisplayName := lpBuffer;
  bi.lpszTitle := pChar(caption);
  bi.ulFlags   := BIF_RETURNONLYFSDIRS;
  bi.lpfn      := nil;
  bi.lParam    := 0;
  pidlBrowse   := SHBrowseForFolder(bi);
  if (pidlBrowse <> nil) then
     begin
       if (SHGetPathFromIDList(pidlBrowse, lpBuffer)) then
          Result := lpBuffer;
     end;
  StrDispose(lpBuffer);
end; {GetFolder}

procedure replaceVariable(sl: TStringList;vari,content: string);
var i,p: integer;
    s: string;
begin
  for i := 0 to sl.Count - 1 do
      begin
        s := sl[i];
        p := pos(vari,s);
        while (p > 0) do
        begin
          s := copy(s,1,p-1) + content + copy(s,p+length(vari),length(s));
          p := pos(vari,s);
        end;
        sl[i] := s;
      end;
end;


function TFormMain.encodeString(s: string): string;
var i: integer;
begin
  if (not encode) then
     begin
       result := s;
       exit;
     end;

  result := '';
  for i := 1 to length(s) do
      if (s[i] = '<') then
         result := result + '&lt;'
      else
      if (s[i] = '>') then
         result := result + '&gt;'
      else
      if (s[i] = '') then
         result := result + '&szlig;'
      else
      if (s[i] = '') then
         result := result + '&auml;'
      else
      if (s[i] = '') then
         result := result + '&ouml;'
      else
      if (s[i] = '') then
         result := result + '&uuml;'
      else
      if (s[i] = '') then
         result := result + '&Auml;'
      else
      if (s[i] = '') then
         result := result + '&Ouml;'
      else
      if (s[i] = '') then
         result := result + '&Uuml;'
      else
         result := result + s[i];
end;


procedure TFormMain.ButtonExitClick(Sender: TObject);
begin
  Close;
end;


function extractName(s: string): string;
begin
  if (copy(ansiuppercase(s),length(s)-3,4) = '.URL') then
     result := copy(s,1,length(s)-4)
  else
     result := s;
end;


function TFormMain.path0(s: string): string;
begin
  result := encodeString(s);
end;

function TFormMain.path1(s: string): string;
begin
  result := encodeString(copy(s,length(FavoritesDirectory)+2,length(s)));
end;

function TFormMain.path2(s: string): string;
var i: integer;
begin
  result := path1(s);
  for i := 1 to length(result) do
      if (result[i] = '\') then
         result[i] := ' ';
end;


(*----------------------------------------------------------------------------*)
(* Read an URL and extract it to sl.                                          *)
(*----------------------------------------------------------------------------*)
procedure TFormMain.ProcessUrl(path,s: string);
var ini: TIniFile;
    url: string;
    Year, Month, Day: Word;
begin
  DecodeDate(Now,Year, Month, Day);
  ini := TIniFile.Create(path + '\' + s);
  try
    url := ini.ReadString('InternetShortcut','URL','???');
    r.Assign(MemoUrlData.Lines);
    replaceVariable(r,'$path0' ,path);
    replaceVariable(r,'$path1' ,path1(path));
    replaceVariable(r,'$path2' ,path2(path));
    replaceVariable(r,'$url'  ,url);
    replaceVariable(r,'$title0',ExtractName(s));
    replaceVariable(r,'$title1',encodeString(ExtractName(s)));
    replaceVariable(r,'$day'  ,IntToStr(day));
    replaceVariable(r,'$month',IntToStr(month));
    replaceVariable(r,'$year' ,IntToStr(year));
    sl.AddStrings(r);
  finally
    ini.Free;
  end;
end;


(*----------------------------------------------------------------------------*)
(* Read a directory                                                           *)
(*----------------------------------------------------------------------------*)
procedure TFormMain.ProcessDir(path,title: string);
var Status: integer;
    srec  : TSearchRec;
    hlis  : TStringList;
    s     : string;
    i     : integer;
begin
  LabelDir.Caption := path;

  hlis := TStringList.Create;
  try
    hlis.Sorted := true;

    r.Assign(MemoFolderHeader.Lines);
    replaceVariable(r,'$title0',title);
    replaceVariable(r,'$title1',encodeString(title));
    sl.AddStrings(r);

    status := findfirst(path + '\*.*', faAnyFile,srec);
    while ((status = 0) and not breakFlag) do
    begin
      if (((srec.attr and faDirectory) <> 0) and
          (srec.name <> '.' )                and
          (srec.name <> '..')
         ) then
         hlis.add('1' + path + '\' + srec.name)
      else
      if ((srec.Name <> '.') and (srec.Name <> '..')) then
         hlis.add('0' + srec.Name);

      Application.ProcessMessages;
      status := findnext(srec);
    end;

    (*--- Now URLs from this level and sublevels are separated by 0 and 1 ---*)
    i := 0;
    while ((i < hlis.Count) and not breakFlag) do
    begin
      s := hlis[i];
      if (s >= '1') then
         ProcessDir(copy(s,2,length(s)),ExtractFileName(s))
      else
         ProcessUrl(path,copy(s,2,length(s)));
      Application.ProcessMessages;
      inc(i);
    end;

    r.Assign(MemoFolderFooter.Lines);
    replaceVariable(r,'$title0',title);
    replaceVariable(r,'$title1',encodeString(title));
    sl.AddStrings(r);
    findclose(srec);
  finally
    hlis.Free;
  end;
end;

procedure TFormMain.ButtonStartClick(Sender: TObject);
begin
  if (breakFlag) then
     begin
       breakFlag := false;
       ButtonStart.Caption := '&Stop';
       sl.Clear;
       sl.AddStrings(MemoHeader.Lines);
       ProcessDir(FavoritesDirectory,extractfilename(FavoritesDirectory));
       sl.AddStrings(MemoFooter.Lines);
       ButtonStart.Caption := '&Generate';
     end
  else
     begin
       breakFlag := true;
     end;
end;

procedure TFormMain.ButtonSaveAsClick(Sender: TObject);
begin
  if (SaveDialog.Execute) then
     sl.SaveToFile(SaveDialog.Filename);
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  FavoritesDirectory := '';
  LabelDir.Caption   := '';
  sl                 := TStringList.Create;
  r                  := TStringList.Create;
  encode             := true;
  breakFlag          := true;
  ReadIniFile;
end;

procedure TFormMain.ButtonSearchFolderClick(Sender: TObject);
begin
//  FavoritesDirectory := GetFolder(CSIDL_Drives,'');
  FavoritesDirectory := GetFolder(CSIDL_FAVORITES,'');
  LabelDir.Caption   := FavoritesDirectory;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  WriteIniFile;
  r.Free;
  sl.Free;
end;

procedure TFormMain.CheckBoxEncodeClick(Sender: TObject);
begin
  encode := CheckBoxEncode.Checked;
end;

function TFormMain.GetIniFileName: string;
var s: string;
begin
  s := lowercase(Application.Exename);
  if (copy(s,length(s)-3,4) = '.exe') then
     result := copy(s,1,length(s)-4) + '.ini'
  else
     raise Exception.Create('Application name expected to end with .exe');
end;

procedure TFormMain.ReadIniFile;
var ini: TIniFile;
    s  : string;
    i  : integer;
begin
  ini := TIniFile.Create(getIniFileName());
  try
    if (scheme = '') then
       begin
         Scheme := ini.ReadString('Common','Scheme','Default');
         LabelCurrentScheme.Caption := Scheme;

         ListboxSchemes.Items.Clear;
         s := '';
         i := 0;
         while (s <> 'END-OF-DATA') do
         begin
           s := ini.ReadString('Schemes',IntToStr(i),'END-OF-DATA');
           if (s <> 'END-OF-DATA') then
              ListboxSchemes.Items.Add(s);
           inc(i);
         end;
       end;

    FavoritesDirectory := ini.ReadString(Scheme,'BaseDir','c:\');

    CheckBoxEncode.Checked := ini.ReadBool(Scheme,'Encode',false);


    MemoHeader.Lines.Clear;
    s := '';
    i := 0;
    while (s <> 'END-OF-DATA') do
    begin
      s := ini.ReadString(Scheme,'MemoHeader' + IntToStr(i),'END-OF-DATA');
      if (s <> 'END-OF-DATA') then
         MemoHeader.Lines.Add(s);
      inc(i);
    end;

    MemoFooter.Lines.Clear;
    s := '';
    i := 0;
    while (s <> 'END-OF-DATA') do
    begin
      s := ini.ReadString(Scheme,'MemoFooter' + IntToStr(i),'END-OF-DATA');
      if (s <> 'END-OF-DATA') then
         MemoFooter.Lines.Add(s);
      inc(i);
    end;

    MemoFolderHeader.Lines.Clear;
    s := '';
    i := 0;
    while (s <> 'END-OF-DATA') do
    begin
      s := ini.ReadString(Scheme,'MemoFolderHeader' + IntToStr(i),'END-OF-DATA');
      if (s <> 'END-OF-DATA') then
         MemoFolderHeader.Lines.Add(s);
      inc(i);
    end;

    MemoFolderFooter.Lines.Clear;
    s := '';
    i := 0;
    while (s <> 'END-OF-DATA') do
    begin
      s := ini.ReadString(Scheme,'MemoFolderFooter' + IntToStr(i),'END-OF-DATA');
      if (s <> 'END-OF-DATA') then
         MemoFolderFooter.Lines.Add(s);
      inc(i);
    end;

    MemoUrlData.Lines.Clear;
    s := '';
    i := 0;
    while (s <> 'END-OF-DATA') do
    begin
      s := ini.ReadString(Scheme,'MemoUrlData' + IntToStr(i),'END-OF-DATA');
      if (s <> 'END-OF-DATA') then
         MemoUrlData.Lines.Add(s);
      inc(i);
    end;

  finally
    ini.Free;
  end;
end;

procedure TFormMain.WriteIniFile;
var ini: TIniFile;
    i  : integer;
begin
  ini := TIniFile.Create(getIniFileName());
  try
    ini.WriteString('Common','Scheme',Scheme);

    for i := 0 to ListboxSchemes.Items.Count - 1 do
        ini.WriteString('Schemes',IntToStr(i),ListboxSchemes.Items[i]);

    if (Scheme <> '') then
       begin
         ini.WriteString(Scheme,'BaseDir',FavoritesDirectory);
         ini.WriteBool(Scheme,'Encode',CheckBoxEncode.Checked);

         for i := 0 to MemoHeader.Lines.Count - 1 do
             ini.WriteString(Scheme,'MemoHeader' + IntToStr(i),MemoHeader.Lines[i]);
         for i := 0 to MemoFooter.Lines.Count - 1 do
             ini.WriteString(Scheme,'MemoFooter' + IntToStr(i),MemoFooter.Lines[i]);
         for i := 0 to MemoFolderHeader.Lines.Count - 1 do
             ini.WriteString(Scheme,'MemoFolderHeader' + IntToStr(i),MemoFolderHeader.Lines[i]);
         for i := 0 to MemoFolderFooter.Lines.Count - 1 do
             ini.WriteString(Scheme,'MemoFolderFooter' + IntToStr(i),MemoFolderFooter.Lines[i]);
         for i := 0 to MemoUrlData.Lines.Count - 1 do
             ini.WriteString(Scheme,'MemoUrlData' + IntToStr(i),MemoUrlData.Lines[i]);
       end;
  finally
    ini.Free;
  end;
end;

procedure TFormMain.DeleteScheme;
var ini: TIniFile;
    i  : integer;
begin
  ini := TIniFile.Create(getIniFileName());
  try
    ini.DeleteKey('Schemes',Scheme);
    ini.EraseSection('Schemes');
    ini.EraseSection(Scheme);
  finally
    ini.Free;
  end;
end;




procedure TFormMain.ButtonAddSchemeClick(Sender: TObject);
var
  s: string;
  sl: TStringList;
  index,i: integer;
begin
  s := InputBox('Input dialog', 'Name for the new scheme', '');
  if (s = '') then
     exit;

  Scheme := Uppercase(s);

  sl := TStringList.Create;
  try
    sl.Sorted := true;
    sl.Assign(ListboxSchemes.Items);
    if (not sl.Find(uppercase(s),index)) then
       ListboxSchemes.Items.Add(uppercase(s));

    try
      ListboxSchemes.Multiselect := true;
      for i := 0 to ListboxSchemes.Items.Count - 1 do
          ListboxSchemes.Selected[i] := false;
      for i := 0 to ListboxSchemes.Items.Count - 1 do
          if (ListboxSchemes.Items[i] = uppercase(s)) then
             ListboxSchemes.Selected[i] := true;
      ListboxSchemes.Multiselect := false;
    except
    end;
  finally
    sl.Free;
  end;
end;

procedure TFormMain.ButtonSaveSchemeClick(Sender: TObject);
begin
  WriteIniFile;
end;

procedure TFormMain.ListBoxSchemesClick(Sender: TObject);
var s: string;
    i: integer;
begin
  WriteIniFile;
  for i := 0 to ListboxSchemes.Items.Count - 1 do
      if (ListboxSchemes.Selected[i]) then
         s := ListboxSchemes.Items[i];
  LabelCurrentScheme.Caption := s;
  Scheme                     := s;
  ReadIniFile;
end;

procedure TFormMain.ButtonDeleteSchemeClick(Sender: TObject);
var i: integer;
begin
  DeleteScheme;
  
  for i := 0 to ListboxSchemes.Items.Count - 1 do
      if (ListBoxSchemes.Selected[i]) then
         begin
           ListBoxSchemes.Items.Delete(i);
           break;
         end;
  Scheme := '';
end;

end.
